package dtu.library.persistency;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dtu.library.app.Medium;
import dtu.library.app.UpdateActivity;
import dtu.library.app.User;

public class PersistencyLayer {

	private static final String USERS_FILENAME = "users.txt";
	private static final String MEDIA_FILENAME = "media.txt";

	private Map<String, PersistentObject> cache_users = new HashMap<String, PersistentObject>();
	private Map<String, PersistentObject> cache_media = new HashMap<String, PersistentObject>();

	public static void clearDatabase() {
		File f = new File(USERS_FILENAME);
		f.delete();
		f = new File(MEDIA_FILENAME);
		f.delete();
	}
	
	public void createUser(User po) throws IOException {
		create(po,USERS_FILENAME,cache_users);
	}

	public void createMedium(Medium po) throws IOException {
		create(po,MEDIA_FILENAME,cache_media);
	}
	
	void create(PersistentObject po, String filename, Map<String,PersistentObject> cache) throws IOException {
		PrintWriter out = new PrintWriter(new FileWriter(filename, true));
		try {
			po.storeOn(out);
			cache.put(po.getKey(), po);
		} finally {
			out.close();
		}
	}

	PersistentObject read(String key,String filename, Map<String,PersistentObject> cache) throws IOException {
		if (cache.containsKey(key)) {
			return cache.get(key);
		}
		try {
			BufferedReader in = new BufferedReader(new FileReader(filename));
			try {
				while (in.ready()) {
					PersistentObject po = PersistentObject.read(in,this);
					if (po.getKey().equals(key)) {
						cache.put(key,po);
						return po;
					}
				}
			} finally {
				in.close();
			}
			return null;
		} catch (FileNotFoundException e) {
			return null;
		}
	}

	public User readUser(String key) throws IOException {
		return (User) read(key,USERS_FILENAME,cache_users);
	}

	public Medium readMedium(String key) throws IOException {
		return (Medium) read(key,MEDIA_FILENAME,cache_media);
	}
	
	public void updateMedium(Medium po1) throws IOException {
		update(po1,MEDIA_FILENAME);
	}

	public void updateUser(User po1) throws IOException {
		update(po1,USERS_FILENAME);
	}

	void update(PersistentObject po1, String filename) throws IOException {
		update(po1, new UpdateActivity<PersistentObject>() {
			public void updateActivity(PrintWriter out, PersistentObject newPo,
					PersistentObject oldPo) {
				if (oldPo.getKey().equals(newPo.getKey())) {
					newPo.storeOn(out);
				} else {
					oldPo.storeOn(out);
				}
			}
		}, filename);
	}

	<T> void update(T newPo, UpdateActivity<T> updateActivity, String filename)
			throws IOException {
		final String tmp_filename = "tmp_" + filename;
		PrintWriter out = new PrintWriter(new FileWriter(tmp_filename, false));
		BufferedReader in = new BufferedReader(new FileReader(filename));
		try {
			while (in.ready()) {
				PersistentObject oldPo = PersistentObject.read(in, this);
				updateActivity.updateActivity(out, newPo, oldPo);
			}
			;
		} finally {
			in.close();
			out.close();
		}
		new File(tmp_filename).renameTo(new File(filename));
	}

	public void delete(String key,String filename,Map<String,PersistentObject> cache) throws IOException {
		update(key, new UpdateActivity<String>() {
			@Override
			public void updateActivity(PrintWriter out, String key,
					PersistentObject po) {
				if (!key.equals(po.getKey())) {
					po.storeOn(out);
				}
			}
		},filename);
		cache.remove(key);
	}

	public void deleteUser(String cpr) throws IOException {
		delete(cpr,USERS_FILENAME,cache_users);
		}

	public void deleteMedium(String signature) throws IOException {
		delete(signature,MEDIA_FILENAME,cache_media);
		}
	
	public List<Medium> getMedia() throws IOException {
		List<Medium> media = new ArrayList<Medium>();
		for (PersistentObject obj : getPersistentObjects(MEDIA_FILENAME)) {
			media.add((Medium) obj);
		}
		return media;
	}

	public List<User> getUsers() throws IOException {
		List<User> media = new ArrayList<User>();
		for (PersistentObject obj : getPersistentObjects(USERS_FILENAME)) {
			media.add((User) obj);
		}
		return media;
	}
	
	List<PersistentObject> getPersistentObjects(String filename) throws IOException {
		List<PersistentObject> result = new ArrayList<PersistentObject>();
			BufferedReader in;
			try {
				in = new BufferedReader(new FileReader(filename));
			} catch (FileNotFoundException e) {
				return result;
			}
			try {
				while (in.ready()) {
					result.add(PersistentObject.read(in, this));
					}
			} finally {
				in.close();
			}
			return result;
	}
}
